/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

#include "common.h"

#include <stdio.h>
#include <pwd.h>

#include "adm.h"
#include "adm_add.h"
#include "adm_check.h"
#include "udb_check.h"

extern char *getpass(char *);

int admuid[]= {ADM_UIDS, -1};


/* SET_HTTPD - SET_CFADM - set our effective uid to be whatever httpd runs as
 * or whatever the conferences are owned by.
 */

void set_httpd() { setuid(SERVER_UID); }
void set_cfadm() { setuid(CFADM_UID); }


/* CHECK_ADM - Check if I am on the list of administrators */

int check_adm(int realuid)
{
int i;

    for (i= 0; admuid[i] >= 0; i++)
	if (realuid == admuid[i])
	    return 1;
    return 0;
}


/* YES - read the answer to a yes/no question from standard input */

int yes()
{
char bf[BFSZ+1];

    if (fgets(bf,BFSZ,stdin) == NULL)
	die("EOF\n");

    return (bf[0] == 'y' || bf[0] == 'Y');
}


/* GETSTR - Read a string with the given maximum length into the given buffer.
 * The buffer should be at least one character bigger than maxlen, to allow
 * room for the terminator.
 */

void getstr(char *bf, int maxlen)
{
int i= 0, ch;

    while ((ch= getchar()) != EOF && ch != '\n')
    {
	if (i < maxlen) bf[i]= ch;
	i++;
    }

    if (ch == EOF) die("EOF\n");
}


/* GETFN - Get a valid fullname.
 */

void getfn(char *fn)
{
char errmesg[BFSZ];

    for(;;)
    {
	printf("Enter your full name: ");
	getstr(fn,BFSZ);
	if (!fullname_bad(fn,errmesg)) break;
	fputs(errmesg,stdout);
    }
}

/* GETPWD - Get a password from the and load the unencrypted copy into the
 * buffer pw.  lid should give the user's selected login name.
 */

void getpwd(char *pw, char *lid)
{
char errmesg[BFSZ];
char *tpw;
int tries;

    for(tries= 0; tries < 3; tries++)
    {
	tpw= getpass("Select password: ");
	if (passwd_bad(tpw,lid,errmesg))
	    fputs(errmesg,stdout);
	else
	{
	    strcpy(pw,tpw);
	    tpw= getpass("Retype password: ");
	    if (!strcmp(tpw,pw))
		return;
	    fputs("The two passwords were not the same!\n",stdout);
	}
    }
    die("Giving up.\n");
}


/* GETUSER - Get a description of the account to link to.  The account will
 * be either a new account to be created, in which case "lid," "fn," and "pw"
 * will have suitable values stored in them and 1 will be returned, or it
 * will be an existing account to link to, in which case only "lid" will be
 * set and a 0 will be returned.  In the call lid should be given an initial
 * guess at the login id to use, and fn may.
 */

int getuser(char *lid, char *fn, char *pw)
{
char *encpw;
char errmesg[BFSZ];

    for (;;)
    {
	if ((encpw= current_passwd(lid)) == NULL)
	{
	    printf("Conference login %s does not exist.\nCreate it? ", lid);
	    if (yes())
	    {
		if (fn[0] == '\0') getfn(fn);
		getpwd(pw,lid);
		return 1;
	    }
	}
	else
	{
#ifdef UID_MAP_FILE
	    printf("Conference login %s exists.\nLink to this account? ", lid);
	    if (yes())
	    {
		if (strcmp(crypt(getpass("Conference password: "),encpw),encpw)
		    die("Password incorrect.\n");
		return 0;
	    }
#else
	    printf("Conference login %s exists.\n");
#endif
	}

	/* Get a valid login name from the user */
	fn[0]= '\0';
	for(;;)
	{
	    printf("Enter another login name: ");
	    getstr(lid,MAX_LOGIN_LEN);
	    if (!login_bad(lid,errmesg)) break;
	    fputs(errmesg,stdout);
	}
    }
}


main(int argc, char **argv)
{
int i;
struct passwd *pw;
int myuid;
char lid[MAX_LOGIN_LEN+1], fn[BFSZ+1], pwd[BFSZ+1], dir[BFSZ+1];

    if (argc > 2)
	die("usage:  %s [<unix-login>]\n",argv[0]);

    umask(022);

    /* Identify the Unix account to be linked */
    if (argc > 1)
    {
	if ((pw= getpwnam(argv[1])) == NULL)
	    die("User %s does not exist.\n",argv[1]);

	myuid= getuid();

	if (pw->pw_uid != myuid && !check_adm(myuid))
	    die("Only conference administrators can link other user's"
		" accounts.\n",argv[1]);
    }
    else
    {
	if ((pw= getpwuid(myuid)) == NULL)
	    die("Could not find you in passwd file.\n");
    }

    /* Save default values */
    strcpy(lid, pw->pw_name);
    strcpy(fn, pw->pw_gecos);

    /* Get user info */
    if (getuser(lid,fn,pwd))
    {
	/* Create conference account */
	adduser(lid, pwd, fn, dir);
    }

#ifdef UID_MAP_FILE
    /* Link conference account to unix account */
    linkuser(myuid,lid);
#endif

    exit(0);
}
